@@ -20,7 +20,6 @@ Device selection options (see: list -v):
   --bus <bus>                    Filter devices by bus
   --address <address>            Filter devices by address in bus
   --usb-port <port>              Filter devices by USB port in bus
-  -d, --device <id>              Select device by listing id
 
 Animation options (devices/modes can support zero or more):
   --speed <value>                Abstract animation speed (device/mode specific)
@@ -28,7 +27,7 @@ Animation options (devices/modes can support zero or more):
   --time-off <value>             Time to wait with the LED turned off (seconds)
   --alert-threshold <number>     Threshold temperature for a visual alert (°C)
   --alert-color <color>          Color used by the visual high temperature alert
-  --direction <string>           If the pattern should move forward or backwards. [default: forward]
+  --direction <string>           If the pattern should move forward or backward.
   --start-led <number>           The first led to start the effect at
   --maximum-leds <number>        The number of LED's the effect should apply to
 
@@ -47,10 +46,11 @@ Other interface options:
   --help                         Show this message
 
 Deprecated:
-  --hid <ignored>                Deprecated
+  -d, --device <index>           Select device by listing index
+  --hid <ignored>                Ignored
 
-Copyright (C) 2018–2021  Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey and
-contributors
+Copyright (C) 2018–2021  Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey, Andrew
+Robertson  and contributors
 
 liquidctl incorporates work by leaty, Ksenija Stanojevic, Alexander Tong, Jens
 Neumaier, Kristóf Jakab, Sean Nelson, Chris Griffith, notaz, realies and Thomas
@@ -64,11 +64,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 """
 
 import datetime
+import errno
 import inspect
 import logging
 import os
 import sys
-import errno
+from traceback import format_exception
 
 from docopt import docopt
 
@@ -77,7 +78,6 @@ from liquidctl.error import NotSupportedByDevice, NotSupportedByDriver, UnsafeFe
 from liquidctl.util import color_from_str
 from liquidctl.version import __version__
 
-
 # conversion from CLI arg to internal option; as options as forwarded to bused
 # and drivers, they must:
 #  - have no default value in the CLI level (not forwarded unless explicitly set);
@@ -93,18 +93,18 @@ _PARSE_ARG = {
     '--match': str,
     '--pick': int,
 
-    '--speed': str,
+    '--speed': str.lower,
     '--time-per-color': int,
     '--time-off': int,
     '--alert-threshold': int,
     '--alert-color': color_from_str,
     '--temperature-sensor': int,
-    '--direction': str,
+    '--direction': str.lower,
     '--start-led': int,
     '--maximum-leds': int,
 
     '--single-12v-ocp': bool,
-    '--pump-mode': str,
+    '--pump-mode': str.lower,
     '--legacy-690lc': bool,
     '--non-volatile': bool,
     '--unsafe': lambda x: x.lower().split(','),
@@ -121,17 +121,19 @@ _FILTER_OPTIONS = [
     'bus',
     'address',
     'usb-port',
+    'match',
     'pick',
     # --device generates no option
 ]
 
 # custom number formats for values of select units
 _VALUE_FORMATS = {
-    '°C': '.1f',
-    'rpm': '.0f',
-    'V': '.2f',
+    '%': '.0f',
     'A': '.2f',
-    'W': '.2f'
+    'V': '.2f',
+    'W': '.2f',
+    'rpm': '.0f',
+    '°C': '.1f',
 }
 
 _LOGGER = logging.getLogger(__name__)
@@ -142,9 +144,9 @@ def _list_devices(devices, using_filters=False, device_id=None, verbose=False, d
         warnings = []
 
         if not using_filters:
-            print(f'Device ID {i}: {dev.description}')
+            print(f'Device #{i}: {dev.description}')
         elif device_id is not None:
-            print(f'Device ID {device_id}: {dev.description}')
+            print(f'Device #{device_id}: {dev.description}')
         else:
             print(f'Result #{i}: {dev.description}')
         if not verbose:
@@ -213,15 +215,15 @@ def _print_dev_status(dev, status):
 
 def _device_set_color(dev, args, **opts):
     color = map(color_from_str, args['<color>'])
-    dev.set_color(args['<channel>'], args['<mode>'], color, **opts)
+    dev.set_color(args['<channel>'].lower(), args['<mode>'].lower(), color, **opts)
 
 
 def _device_set_speed(dev, args, **opts):
     if len(args['<temperature>']) > 0:
         profile = zip(map(int, args['<temperature>']), map(int, args['<percentage>']))
-        dev.set_speed_profile(args['<channel>'], profile, **opts)
+        dev.set_speed_profile(args['<channel>'].lower(), profile, **opts)
     else:
-        dev.set_fixed_speed(args['<channel>'], int(args['<percentage>'][0]), **opts)
+        dev.set_fixed_speed(args['<channel>'].lower(), int(args['<percentage>'][0]), **opts)
 
 
 def _make_opts(args):
@@ -277,16 +279,17 @@ def main():
     if not args['--device']:
         selected = list(find_liquidctl_devices(**opts))
     else:
+        _LOGGER.warning('-d/--device is deprecated, prefer --match or other selection options')
         device_id = int(args['--device'])
         no_filters = {opt: val for opt, val in opts.items() if opt not in _FILTER_OPTIONS}
         compat = list(find_liquidctl_devices(**no_filters))
         if device_id < 0 or device_id >= len(compat):
-            raise SystemExit('Error: device ID out of bounds')
+            raise SystemExit('Error: device index out of bounds')
         if filter_count:
             # check that --device matches other filter criteria
             matched_devs = [dev.device for dev in find_liquidctl_devices(**opts)]
             if compat[device_id].device not in matched_devs:
-                raise SystemExit('Error: device ID does not match remaining selection criteria')
+                raise SystemExit('Error: device index does not match remaining selection criteria')
             _LOGGER.warning('mixing --device <id> with other filters is not recommended; '
                             'to disambiguate between results prefer --pick <result>')
         selected = [compat[device_id]]
@@ -302,26 +305,30 @@ def main():
 
     errors = 0
 
-    def log_error(err, msg, *args):
+    def log_error(err, msg, append_err=False, *args):
         nonlocal errors
         errors += 1
         _LOGGER.info('%s', err, exc_info=True)
-        _LOGGER.error(msg, *args)
+        if append_err:
+            exception = list(format_exception(Exception, err, None))[-1].rstrip()
+            _LOGGER.error(f'{msg}: {exception}', *args)
+        else:
+            _LOGGER.error(msg, *args)
 
     for dev in selected:
         _LOGGER.debug('device: %s', dev.description)
         try:
-            dev.connect(**opts)
-            if args['initialize']:
-                _print_dev_status(dev, dev.initialize(**opts))
-            elif args['status']:
-                _print_dev_status(dev, dev.get_status(**opts))
-            elif args['set'] and args['speed']:
-                _device_set_speed(dev, args, **opts)
-            elif args['set'] and args['color']:
-                _device_set_color(dev, args, **opts)
-            else:
-                raise Exception('not sure what to do')
+            with dev.connect(**opts):
+                if args['initialize']:
+                    _print_dev_status(dev, dev.initialize(**opts))
+                elif args['status']:
+                    _print_dev_status(dev, dev.get_status(**opts))
+                elif args['set'] and args['speed']:
+                    _device_set_speed(dev, args, **opts)
+                elif args['set'] and args['color']:
+                    _device_set_color(dev, args, **opts)
+                else:
+                    assert False, 'unreachable'
         except OSError as err:
             # each backend API returns a different subtype of OSError (OSError,
             # usb.core.USBError or PermissionError) for permission issues
@@ -330,7 +337,7 @@ def main():
             elif err.args == ('open failed', ):
                 log_error(err, f'Error: could not open {dev.description}, possibly due to insufficient permissions')
             else:
-                log_error(err, f'Unexpected OS error with {dev.description}: {err}')
+                log_error(err, f'Unexpected OS error with {dev.description}', append_err=True)
         except NotSupportedByDevice as err:
             log_error(err, f'Error: operation not supported by {dev.description}')
         except NotSupportedByDriver as err:
@@ -340,9 +347,7 @@ def main():
             log_error(err, f'Error: missing --unsafe features for {dev.description}: {features!r}')
             _LOGGER.error('More information is provided in the corresponding device guide')
         except Exception as err:
-            log_error(err, f'Unexpected error with {dev.description}: {err}')
-        finally:
-            dev.disconnect(**opts)
+            log_error(err, f'Unexpected error with {dev.description}', append_err=True)
 
     if errors:
         sys.exit(errors)
